@@ -1,20 +1,19 @@
 import pytest
-
 import redis
-from redis import exceptions
+from redis import Redis, exceptions
 from redis.commands.json.decoders import decode_list, unstring
 from redis.commands.json.path import Path
 
-from .conftest import skip_ifmodversion_lt
+from .conftest import _get_client, assert_resp_response, skip_ifmodversion_lt
 
 
 @pytest.fixture
-def client(modclient):
-    modclient.flushdb()
-    return modclient
+def client(request):
+    r = _get_client(Redis, request, decode_responses=True)
+    r.flushdb()
+    return r
 
 
-@pytest.mark.redismod
 def test_json_setbinarykey(client):
     d = {"hello": "world", b"some": "value"}
     with pytest.raises(TypeError):
@@ -22,33 +21,29 @@ def test_json_setbinarykey(client):
     assert client.json().set("somekey", Path.root_path(), d, decode_keys=True)
 
 
-@pytest.mark.redismod
 def test_json_setgetdeleteforget(client):
     assert client.json().set("foo", Path.root_path(), "bar")
-    assert client.json().get("foo") == "bar"
+    assert_resp_response(client, client.json().get("foo"), "bar", [["bar"]])
     assert client.json().get("baz") is None
     assert client.json().delete("foo") == 1
     assert client.json().forget("foo") == 0  # second delete
     assert client.exists("foo") == 0
 
 
-@pytest.mark.redismod
 def test_jsonget(client):
     client.json().set("foo", Path.root_path(), "bar")
-    assert client.json().get("foo") == "bar"
+    assert_resp_response(client, client.json().get("foo"), "bar", [["bar"]])
 
 
-@pytest.mark.redismod
 def test_json_get_jset(client):
     assert client.json().set("foo", Path.root_path(), "bar")
-    assert "bar" == client.json().get("foo")
+    assert_resp_response(client, client.json().get("foo"), "bar", [["bar"]])
     assert client.json().get("baz") is None
     assert 1 == client.json().delete("foo")
     assert client.exists("foo") == 0
 
 
-@pytest.mark.redismod
-@skip_ifmodversion_lt("2.6.0", "ReJSON")  # todo: update after the release
+@skip_ifmodversion_lt("2.06.00", "ReJSON")  # todo: update after the release
 def test_json_merge(client):
     # Test with root path $
     assert client.json().set(
@@ -80,15 +75,16 @@ def test_json_merge(client):
     }
 
 
-@pytest.mark.redismod
 def test_nonascii_setgetdelete(client):
     assert client.json().set("notascii", Path.root_path(), "hyvää-élève")
-    assert "hyvää-élève" == client.json().get("notascii", no_escape=True)
+    res = "hyvää-élève"
+    assert_resp_response(
+        client, client.json().get("notascii", no_escape=True), res, [[res]]
+    )
     assert 1 == client.json().delete("notascii")
     assert client.exists("notascii") == 0
 
 
-@pytest.mark.redismod
 def test_jsonsetexistentialmodifiersshouldsucceed(client):
     obj = {"foo": "bar"}
     assert client.json().set("obj", Path.root_path(), obj)
@@ -106,7 +102,6 @@ def test_jsonsetexistentialmodifiersshouldsucceed(client):
         client.json().set("obj", Path("foo"), "baz", nx=True, xx=True)
 
 
-@pytest.mark.redismod
 def test_mgetshouldsucceed(client):
     client.json().set("1", Path.root_path(), 1)
     client.json().set("2", Path.root_path(), 2)
@@ -115,8 +110,8 @@ def test_mgetshouldsucceed(client):
     assert client.json().mget([1, 2], Path.root_path()) == [1, 2]
 
 
-@pytest.mark.redismod
-@skip_ifmodversion_lt("2.6.0", "ReJSON")  # todo: update after the release
+@pytest.mark.onlynoncluster
+@skip_ifmodversion_lt("2.06.00", "ReJSON")
 def test_mset(client):
     client.json().mset([("1", Path.root_path(), 1), ("2", Path.root_path(), 2)])
 
@@ -124,40 +119,49 @@ def test_mset(client):
     assert client.json().mget(["1", "2"], Path.root_path()) == [1, 2]
 
 
-@pytest.mark.redismod
 @skip_ifmodversion_lt("99.99.99", "ReJSON")  # todo: update after the release
 def test_clear(client):
     client.json().set("arr", Path.root_path(), [0, 1, 2, 3, 4])
     assert 1 == client.json().clear("arr", Path.root_path())
-    assert [] == client.json().get("arr")
+    assert_resp_response(client, client.json().get("arr"), [], [[[]]])
 
 
-@pytest.mark.redismod
 def test_type(client):
     client.json().set("1", Path.root_path(), 1)
-    assert "integer" == client.json().type("1", Path.root_path())
-    assert "integer" == client.json().type("1")
+    assert_resp_response(
+        client, client.json().type("1", Path.root_path()), "integer", ["integer"]
+    )
+    assert_resp_response(client, client.json().type("1"), "integer", ["integer"])
 
 
-@pytest.mark.redismod
 def test_numincrby(client):
     client.json().set("num", Path.root_path(), 1)
-    assert 2 == client.json().numincrby("num", Path.root_path(), 1)
-    assert 2.5 == client.json().numincrby("num", Path.root_path(), 0.5)
-    assert 1.25 == client.json().numincrby("num", Path.root_path(), -1.25)
+    assert_resp_response(
+        client, client.json().numincrby("num", Path.root_path(), 1), 2, [2]
+    )
+    assert_resp_response(
+        client, client.json().numincrby("num", Path.root_path(), 0.5), 2.5, [2.5]
+    )
+    assert_resp_response(
+        client, client.json().numincrby("num", Path.root_path(), -1.25), 1.25, [1.25]
+    )
 
 
-@pytest.mark.redismod
 def test_nummultby(client):
     client.json().set("num", Path.root_path(), 1)
 
     with pytest.deprecated_call():
-        assert 2 == client.json().nummultby("num", Path.root_path(), 2)
-        assert 5 == client.json().nummultby("num", Path.root_path(), 2.5)
-        assert 2.5 == client.json().nummultby("num", Path.root_path(), 0.5)
+        assert_resp_response(
+            client, client.json().nummultby("num", Path.root_path(), 2), 2, [2]
+        )
+        assert_resp_response(
+            client, client.json().nummultby("num", Path.root_path(), 2.5), 5, [5]
+        )
+        assert_resp_response(
+            client, client.json().nummultby("num", Path.root_path(), 0.5), 2.5, [2.5]
+        )
 
 
-@pytest.mark.redismod
 @skip_ifmodversion_lt("99.99.99", "ReJSON")  # todo: update after the release
 def test_toggle(client):
     client.json().set("bool", Path.root_path(), False)
@@ -169,15 +173,15 @@ def test_toggle(client):
         client.json().toggle("num", Path.root_path())
 
 
-@pytest.mark.redismod
 def test_strappend(client):
     client.json().set("jsonkey", Path.root_path(), "foo")
     assert 6 == client.json().strappend("jsonkey", "bar")
-    assert "foobar" == client.json().get("jsonkey", Path.root_path())
+    assert_resp_response(
+        client, client.json().get("jsonkey", Path.root_path()), "foobar", [["foobar"]]
+    )
 
 
-# @pytest.mark.redismod
-# def test_debug(client):
+# # def test_debug(client):
 #    client.json().set("str", Path.root_path(), "foo")
 #    assert 24 == client.json().debug("MEMORY", "str", Path.root_path())
 #    assert 24 == client.json().debug("MEMORY", "str")
@@ -186,7 +190,6 @@ def test_strappend(client):
 #    assert isinstance(client.json().debug("HELP"), list)
 
 
-@pytest.mark.redismod
 def test_strlen(client):
     client.json().set("str", Path.root_path(), "foo")
     assert 3 == client.json().strlen("str", Path.root_path())
@@ -195,7 +198,6 @@ def test_strlen(client):
     assert 6 == client.json().strlen("str")
 
 
-@pytest.mark.redismod
 def test_arrappend(client):
     client.json().set("arr", Path.root_path(), [1])
     assert 2 == client.json().arrappend("arr", Path.root_path(), 2)
@@ -203,7 +205,6 @@ def test_arrappend(client):
     assert 7 == client.json().arrappend("arr", Path.root_path(), *[5, 6, 7])
 
 
-@pytest.mark.redismod
 def test_arrindex(client):
     client.json().set("arr", Path.root_path(), [0, 1, 2, 3, 4])
     assert 1 == client.json().arrindex("arr", Path.root_path(), 1)
@@ -215,19 +216,19 @@ def test_arrindex(client):
     assert -1 == client.json().arrindex("arr", Path.root_path(), 4, start=1, stop=3)
 
 
-@pytest.mark.redismod
 def test_arrinsert(client):
     client.json().set("arr", Path.root_path(), [0, 4])
     assert 5 - -client.json().arrinsert("arr", Path.root_path(), 1, *[1, 2, 3])
-    assert [0, 1, 2, 3, 4] == client.json().get("arr")
+    res = [0, 1, 2, 3, 4]
+    assert_resp_response(client, client.json().get("arr"), res, [[res]])
 
     # test prepends
     client.json().set("val2", Path.root_path(), [5, 6, 7, 8, 9])
     client.json().arrinsert("val2", Path.root_path(), 0, ["some", "thing"])
-    assert client.json().get("val2") == [["some", "thing"], 5, 6, 7, 8, 9]
+    res = [["some", "thing"], 5, 6, 7, 8, 9]
+    assert_resp_response(client, client.json().get("val2"), res, [[res]])
 
 
-@pytest.mark.redismod
 def test_arrlen(client):
     client.json().set("arr", Path.root_path(), [0, 1, 2, 3, 4])
     assert 5 == client.json().arrlen("arr", Path.root_path())
@@ -235,14 +236,13 @@ def test_arrlen(client):
     assert client.json().arrlen("fakekey") is None
 
 
-@pytest.mark.redismod
 def test_arrpop(client):
     client.json().set("arr", Path.root_path(), [0, 1, 2, 3, 4])
     assert 4 == client.json().arrpop("arr", Path.root_path(), 4)
     assert 3 == client.json().arrpop("arr", Path.root_path(), -1)
     assert 2 == client.json().arrpop("arr", Path.root_path())
     assert 0 == client.json().arrpop("arr", Path.root_path(), 0)
-    assert [1] == client.json().get("arr")
+    assert_resp_response(client, client.json().get("arr"), [1], [[[1]]])
 
     # test out of bounds
     client.json().set("arr", Path.root_path(), [0, 1, 2, 3, 4])
@@ -253,11 +253,10 @@ def test_arrpop(client):
     assert client.json().arrpop("arr") is None
 
 
-@pytest.mark.redismod
 def test_arrtrim(client):
     client.json().set("arr", Path.root_path(), [0, 1, 2, 3, 4])
     assert 3 == client.json().arrtrim("arr", Path.root_path(), 1, 3)
-    assert [1, 2, 3] == client.json().get("arr")
+    assert_resp_response(client, client.json().get("arr"), [1, 2, 3], [[[1, 2, 3]]])
 
     # <0 test, should be 0 equivalent
     client.json().set("arr", Path.root_path(), [0, 1, 2, 3, 4])
@@ -276,7 +275,6 @@ def test_arrtrim(client):
     assert 0 == client.json().arrtrim("arr", Path.root_path(), 9, 11)
 
 
-@pytest.mark.redismod
 def test_resp(client):
     obj = {"foo": "bar", "baz": 1, "qaz": True}
     client.json().set("obj", Path.root_path(), obj)
@@ -286,7 +284,6 @@ def test_resp(client):
     assert isinstance(client.json().resp("obj"), list)
 
 
-@pytest.mark.redismod
 def test_objkeys(client):
     obj = {"foo": "bar", "baz": "qaz"}
     client.json().set("obj", Path.root_path(), obj)
@@ -303,7 +300,6 @@ def test_objkeys(client):
     assert client.json().objkeys("fakekey") is None
 
 
-@pytest.mark.redismod
 def test_objlen(client):
     obj = {"foo": "bar", "baz": "qaz"}
     client.json().set("obj", Path.root_path(), obj)
@@ -313,13 +309,12 @@ def test_objlen(client):
     assert len(obj) == client.json().objlen("obj")
 
 
-@pytest.mark.redismod
 def test_json_commands_in_pipeline(client):
     p = client.json().pipeline()
     p.set("foo", Path.root_path(), "bar")
     p.get("foo")
     p.delete("foo")
-    assert [True, "bar", 1] == p.execute()
+    assert_resp_response(client, p.execute(), [True, "bar", 1], [True, [["bar"]], 1])
     assert client.keys() == []
     assert client.get("foo") is None
 
@@ -332,24 +327,23 @@ def test_json_commands_in_pipeline(client):
         p.jsonget("foo")
     p.exists("notarealkey")
     p.delete("foo")
-    assert [True, d, 0, 1] == p.execute()
+    assert_resp_response(client, p.execute(), [True, d, 0, 1], [True, [[d]], 0, 1])
     assert client.keys() == []
     assert client.get("foo") is None
 
 
-@pytest.mark.redismod
 def test_json_delete_with_dollar(client):
     doc1 = {"a": 1, "nested": {"a": 2, "b": 3}}
     assert client.json().set("doc1", "$", doc1)
     assert client.json().delete("doc1", "$..a") == 2
-    r = client.json().get("doc1", "$")
-    assert r == [{"nested": {"b": 3}}]
+    res = [{"nested": {"b": 3}}]
+    assert_resp_response(client, client.json().get("doc1", "$"), res, [res])
 
     doc2 = {"a": {"a": 2, "b": 3}, "b": ["a", "b"], "nested": {"b": [True, "a", "b"]}}
     assert client.json().set("doc2", "$", doc2)
     assert client.json().delete("doc2", "$..a") == 1
-    res = client.json().get("doc2", "$")
-    assert res == [{"nested": {"b": [True, "a", "b"]}, "b": ["a", "b"]}]
+    res = [{"nested": {"b": [True, "a", "b"]}, "b": ["a", "b"]}]
+    assert_resp_response(client, client.json().get("doc2", "$"), res, [res])
 
     doc3 = [
         {
@@ -380,8 +374,7 @@ def test_json_delete_with_dollar(client):
             }
         ]
     ]
-    res = client.json().get("doc3", "$")
-    assert res == doc3val
+    assert_resp_response(client, client.json().get("doc3", "$"), doc3val, [doc3val])
 
     # Test default path
     assert client.json().delete("doc3") == 1
@@ -390,19 +383,18 @@ def test_json_delete_with_dollar(client):
     client.json().delete("not_a_document", "..a")
 
 
-@pytest.mark.redismod
 def test_json_forget_with_dollar(client):
     doc1 = {"a": 1, "nested": {"a": 2, "b": 3}}
     assert client.json().set("doc1", "$", doc1)
     assert client.json().forget("doc1", "$..a") == 2
-    r = client.json().get("doc1", "$")
-    assert r == [{"nested": {"b": 3}}]
+    res = [{"nested": {"b": 3}}]
+    assert_resp_response(client, client.json().get("doc1", "$"), res, [res])
 
     doc2 = {"a": {"a": 2, "b": 3}, "b": ["a", "b"], "nested": {"b": [True, "a", "b"]}}
     assert client.json().set("doc2", "$", doc2)
     assert client.json().forget("doc2", "$..a") == 1
-    res = client.json().get("doc2", "$")
-    assert res == [{"nested": {"b": [True, "a", "b"]}, "b": ["a", "b"]}]
+    res = [{"nested": {"b": [True, "a", "b"]}, "b": ["a", "b"]}]
+    assert_resp_response(client, client.json().get("doc2", "$"), res, [res])
 
     doc3 = [
         {
@@ -433,8 +425,7 @@ def test_json_forget_with_dollar(client):
             }
         ]
     ]
-    res = client.json().get("doc3", "$")
-    assert res == doc3val
+    assert_resp_response(client, client.json().get("doc3", "$"), doc3val, [doc3val])
 
     # Test default path
     assert client.json().forget("doc3") == 1
@@ -443,7 +434,6 @@ def test_json_forget_with_dollar(client):
     client.json().forget("not_a_document", "..a")
 
 
-@pytest.mark.redismod
 def test_json_mget_dollar(client):
     # Test mget with multi paths
     client.json().set(
@@ -457,8 +447,10 @@ def test_json_mget_dollar(client):
         {"a": 4, "b": 5, "nested": {"a": 6}, "c": None, "nested2": {"a": [None]}},
     )
     # Compare also to single JSON.GET
-    assert client.json().get("doc1", "$..a") == [1, 3, None]
-    assert client.json().get("doc2", "$..a") == [4, 6, [None]]
+    res = [1, 3, None]
+    assert_resp_response(client, client.json().get("doc1", "$..a"), res, [res])
+    res = [4, 6, [None]]
+    assert_resp_response(client, client.json().get("doc2", "$..a"), res, [res])
 
     # Test mget with single path
     client.json().mget("doc1", "$..a") == [1, 3, None]
@@ -471,9 +463,7 @@ def test_json_mget_dollar(client):
     assert res == [None, None]
 
 
-@pytest.mark.redismod
 def test_numby_commands_dollar(client):
-
     # Test NUMINCRBY
     client.json().set("doc1", "$", {"a": "b", "b": [{"a": 2}, {"a": 5.0}, {"a": "c"}]})
     # Test multi
@@ -516,24 +506,21 @@ def test_numby_commands_dollar(client):
         client.json().nummultby("doc1", ".b[0].a", 3) == 6
 
 
-@pytest.mark.redismod
 def test_strappend_dollar(client):
-
     client.json().set(
         "doc1", "$", {"a": "foo", "nested1": {"a": "hello"}, "nested2": {"a": 31}}
     )
     # Test multi
     client.json().strappend("doc1", "bar", "$..a") == [6, 8, None]
 
-    client.json().get("doc1", "$") == [
-        {"a": "foobar", "nested1": {"a": "hellobar"}, "nested2": {"a": 31}}
-    ]
+    # res = [{"a": "foobar", "nested1": {"a": "hellobar"}, "nested2": {"a": 31}}]
+    # assert_resp_response(client, client.json().get("doc1", "$"), res, [res])
+
     # Test single
     client.json().strappend("doc1", "baz", "$.nested1.a") == [11]
 
-    client.json().get("doc1", "$") == [
-        {"a": "foobar", "nested1": {"a": "hellobarbaz"}, "nested2": {"a": 31}}
-    ]
+    # res = [{"a": "foobar", "nested1": {"a": "hellobarbaz"}, "nested2": {"a": 31}}]
+    # assert_resp_response(client, client.json().get("doc1", "$"), res, [res])
 
     # Test missing key
     with pytest.raises(exceptions.ResponseError):
@@ -541,18 +528,15 @@ def test_strappend_dollar(client):
 
     # Test multi
     client.json().strappend("doc1", "bar", ".*.a") == 8
-    client.json().get("doc1", "$") == [
-        {"a": "foo", "nested1": {"a": "hellobar"}, "nested2": {"a": 31}}
-    ]
+    # res = [{"a": "foo", "nested1": {"a": "hellobar"}, "nested2": {"a": 31}}]
+    # assert_resp_response(client, client.json().get("doc1", "$"), res, [res])
 
     # Test missing path
     with pytest.raises(exceptions.ResponseError):
         client.json().strappend("doc1", "piu")
 
 
-@pytest.mark.redismod
 def test_strlen_dollar(client):
-
     # Test multi
     client.json().set(
         "doc1", "$", {"a": "foo", "nested1": {"a": "hello"}, "nested2": {"a": 31}}
@@ -572,7 +556,6 @@ def test_strlen_dollar(client):
         client.json().strlen("non_existing_doc", "$..a")
 
 
-@pytest.mark.redismod
 def test_arrappend_dollar(client):
     client.json().set(
         "doc1",
@@ -585,23 +568,25 @@ def test_arrappend_dollar(client):
     )
     # Test multi
     client.json().arrappend("doc1", "$..a", "bar", "racuda") == [3, 5, None]
-    assert client.json().get("doc1", "$") == [
+    res = [
         {
             "a": ["foo", "bar", "racuda"],
             "nested1": {"a": ["hello", None, "world", "bar", "racuda"]},
             "nested2": {"a": 31},
         }
     ]
+    assert_resp_response(client, client.json().get("doc1", "$"), res, [res])
 
     # Test single
     assert client.json().arrappend("doc1", "$.nested1.a", "baz") == [6]
-    assert client.json().get("doc1", "$") == [
+    res = [
         {
             "a": ["foo", "bar", "racuda"],
             "nested1": {"a": ["hello", None, "world", "bar", "racuda", "baz"]},
             "nested2": {"a": 31},
         }
     ]
+    assert_resp_response(client, client.json().get("doc1", "$"), res, [res])
 
     # Test missing key
     with pytest.raises(exceptions.ResponseError):
@@ -620,29 +605,31 @@ def test_arrappend_dollar(client):
     # Test multi (all paths are updated, but return result of last path)
     assert client.json().arrappend("doc1", "..a", "bar", "racuda") == 5
 
-    assert client.json().get("doc1", "$") == [
+    res = [
         {
             "a": ["foo", "bar", "racuda"],
             "nested1": {"a": ["hello", None, "world", "bar", "racuda"]},
             "nested2": {"a": 31},
         }
     ]
+    assert_resp_response(client, client.json().get("doc1", "$"), res, [res])
+
     # Test single
     assert client.json().arrappend("doc1", ".nested1.a", "baz") == 6
-    assert client.json().get("doc1", "$") == [
+    res = [
         {
             "a": ["foo", "bar", "racuda"],
             "nested1": {"a": ["hello", None, "world", "bar", "racuda", "baz"]},
             "nested2": {"a": 31},
         }
     ]
+    assert_resp_response(client, client.json().get("doc1", "$"), res, [res])
 
     # Test missing key
     with pytest.raises(exceptions.ResponseError):
         client.json().arrappend("non_existing_doc", "$..a")
 
 
-@pytest.mark.redismod
 def test_arrinsert_dollar(client):
     client.json().set(
         "doc1",
@@ -656,31 +643,32 @@ def test_arrinsert_dollar(client):
     # Test multi
     assert client.json().arrinsert("doc1", "$..a", "1", "bar", "racuda") == [3, 5, None]
 
-    assert client.json().get("doc1", "$") == [
+    res = [
         {
             "a": ["foo", "bar", "racuda"],
             "nested1": {"a": ["hello", "bar", "racuda", None, "world"]},
             "nested2": {"a": 31},
         }
     ]
+    assert_resp_response(client, client.json().get("doc1", "$"), res, [res])
+
     # Test single
     assert client.json().arrinsert("doc1", "$.nested1.a", -2, "baz") == [6]
-    assert client.json().get("doc1", "$") == [
+    res = [
         {
             "a": ["foo", "bar", "racuda"],
             "nested1": {"a": ["hello", "bar", "racuda", "baz", None, "world"]},
             "nested2": {"a": 31},
         }
     ]
+    assert_resp_response(client, client.json().get("doc1", "$"), res, [res])
 
     # Test missing key
     with pytest.raises(exceptions.ResponseError):
         client.json().arrappend("non_existing_doc", "$..a")
 
 
-@pytest.mark.redismod
 def test_arrlen_dollar(client):
-
     client.json().set(
         "doc1",
         "$",
@@ -728,7 +716,6 @@ def test_arrlen_dollar(client):
     assert client.json().arrlen("non_existing_doc", "..a") is None
 
 
-@pytest.mark.redismod
 def test_arrpop_dollar(client):
     client.json().set(
         "doc1",
@@ -743,9 +730,8 @@ def test_arrpop_dollar(client):
     # # # Test multi
     assert client.json().arrpop("doc1", "$..a", 1) == ['"foo"', None, None]
 
-    assert client.json().get("doc1", "$") == [
-        {"a": [], "nested1": {"a": ["hello", "world"]}, "nested2": {"a": 31}}
-    ]
+    res = [{"a": [], "nested1": {"a": ["hello", "world"]}, "nested2": {"a": 31}}]
+    assert_resp_response(client, client.json().get("doc1", "$"), res, [res])
 
     # Test missing key
     with pytest.raises(exceptions.ResponseError):
@@ -763,18 +749,15 @@ def test_arrpop_dollar(client):
     )
     # Test multi (all paths are updated, but return result of last path)
     client.json().arrpop("doc1", "..a", "1") is None
-    assert client.json().get("doc1", "$") == [
-        {"a": [], "nested1": {"a": ["hello", "world"]}, "nested2": {"a": 31}}
-    ]
+    res = [{"a": [], "nested1": {"a": ["hello", "world"]}, "nested2": {"a": 31}}]
+    assert_resp_response(client, client.json().get("doc1", "$"), res, [res])
 
     # # Test missing key
     with pytest.raises(exceptions.ResponseError):
         client.json().arrpop("non_existing_doc", "..a")
 
 
-@pytest.mark.redismod
 def test_arrtrim_dollar(client):
-
     client.json().set(
         "doc1",
         "$",
@@ -786,19 +769,17 @@ def test_arrtrim_dollar(client):
     )
     # Test multi
     assert client.json().arrtrim("doc1", "$..a", "1", -1) == [0, 2, None]
-    assert client.json().get("doc1", "$") == [
-        {"a": [], "nested1": {"a": [None, "world"]}, "nested2": {"a": 31}}
-    ]
+    res = [{"a": [], "nested1": {"a": [None, "world"]}, "nested2": {"a": 31}}]
+    assert_resp_response(client, client.json().get("doc1", "$"), res, [res])
 
     assert client.json().arrtrim("doc1", "$..a", "1", "1") == [0, 1, None]
-    assert client.json().get("doc1", "$") == [
-        {"a": [], "nested1": {"a": ["world"]}, "nested2": {"a": 31}}
-    ]
+    res = [{"a": [], "nested1": {"a": ["world"]}, "nested2": {"a": 31}}]
+    assert_resp_response(client, client.json().get("doc1", "$"), res, [res])
+
     # Test single
     assert client.json().arrtrim("doc1", "$.nested1.a", 1, 0) == [0]
-    assert client.json().get("doc1", "$") == [
-        {"a": [], "nested1": {"a": []}, "nested2": {"a": 31}}
-    ]
+    res = [{"a": [], "nested1": {"a": []}, "nested2": {"a": 31}}]
+    assert_resp_response(client, client.json().get("doc1", "$"), res, [res])
 
     # Test missing key
     with pytest.raises(exceptions.ResponseError):
@@ -820,16 +801,14 @@ def test_arrtrim_dollar(client):
 
     # Test single
     assert client.json().arrtrim("doc1", ".nested1.a", "1", "1") == 1
-    assert client.json().get("doc1", "$") == [
-        {"a": [], "nested1": {"a": ["world"]}, "nested2": {"a": 31}}
-    ]
+    res = [{"a": [], "nested1": {"a": ["world"]}, "nested2": {"a": 31}}]
+    assert_resp_response(client, client.json().get("doc1", "$"), res, [res])
 
     # Test missing key
     with pytest.raises(exceptions.ResponseError):
         client.json().arrtrim("non_existing_doc", "..a", 1, 1)
 
 
-@pytest.mark.redismod
 def test_objkeys_dollar(client):
     client.json().set(
         "doc1",
@@ -859,7 +838,6 @@ def test_objkeys_dollar(client):
     assert client.json().objkeys("doc1", "$..nowhere") == []
 
 
-@pytest.mark.redismod
 def test_objlen_dollar(client):
     client.json().set(
         "doc1",
@@ -895,7 +873,6 @@ def test_objlen_dollar(client):
     client.json().objlen("doc1", ".nowhere")
 
 
-@pytest.mark.redismod
 def load_types_data(nested_key_name):
     td = {
         "object": {},
@@ -915,21 +892,23 @@ def load_types_data(nested_key_name):
     return jdata, types
 
 
-@pytest.mark.redismod
 def test_type_dollar(client):
     jdata, jtypes = load_types_data("a")
     client.json().set("doc1", "$", jdata)
     # Test multi
-    assert client.json().type("doc1", "$..a") == jtypes
+    assert_resp_response(client, client.json().type("doc1", "$..a"), jtypes, [jtypes])
 
     # Test single
-    assert client.json().type("doc1", "$.nested2.a") == [jtypes[1]]
+    assert_resp_response(
+        client, client.json().type("doc1", "$.nested2.a"), [jtypes[1]], [[jtypes[1]]]
+    )
 
     # Test missing key
-    assert client.json().type("non_existing_doc", "..a") is None
+    assert_resp_response(
+        client, client.json().type("non_existing_doc", "..a"), None, [None]
+    )
 
 
-@pytest.mark.redismod
 def test_clear_dollar(client):
     client.json().set(
         "doc1",
@@ -944,9 +923,10 @@ def test_clear_dollar(client):
     # Test multi
     assert client.json().clear("doc1", "$..a") == 3
 
-    assert client.json().get("doc1", "$") == [
+    res = [
         {"nested1": {"a": {}}, "a": [], "nested2": {"a": "claro"}, "nested3": {"a": {}}}
     ]
+    assert_resp_response(client, client.json().get("doc1", "$"), res, [res])
 
     # Test single
     client.json().set(
@@ -960,7 +940,7 @@ def test_clear_dollar(client):
         },
     )
     assert client.json().clear("doc1", "$.nested1.a") == 1
-    assert client.json().get("doc1", "$") == [
+    res = [
         {
             "nested1": {"a": {}},
             "a": ["foo"],
@@ -968,17 +948,17 @@ def test_clear_dollar(client):
             "nested3": {"a": {"baz": 50}},
         }
     ]
+    assert_resp_response(client, client.json().get("doc1", "$"), res, [res])
 
     # Test missing path (defaults to root)
     assert client.json().clear("doc1") == 1
-    assert client.json().get("doc1", "$") == [{}]
+    assert_resp_response(client, client.json().get("doc1", "$"), [{}], [[{}]])
 
     # Test missing key
     with pytest.raises(exceptions.ResponseError):
         client.json().clear("non_existing_doc", "$..a")
 
 
-@pytest.mark.redismod
 def test_toggle_dollar(client):
     client.json().set(
         "doc1",
@@ -992,7 +972,7 @@ def test_toggle_dollar(client):
     )
     # Test multi
     assert client.json().toggle("doc1", "$..a") == [None, 1, None, 0]
-    assert client.json().get("doc1", "$") == [
+    res = [
         {
             "a": ["foo"],
             "nested1": {"a": True},
@@ -1000,14 +980,14 @@ def test_toggle_dollar(client):
             "nested3": {"a": False},
         }
     ]
+    assert_resp_response(client, client.json().get("doc1", "$"), res, [res])
 
     # Test missing key
     with pytest.raises(exceptions.ResponseError):
         client.json().toggle("non_existing_doc", "$..a")
 
 
-# @pytest.mark.redismod
-# def test_debug_dollar(client):
+# # def test_debug_dollar(client):
 #
 #    jdata, jtypes = load_types_data("a")
 #
@@ -1029,9 +1009,7 @@ def test_toggle_dollar(client):
 #    assert client.json().debug("MEMORY", "non_existing_doc", "$..a") == []
 
 
-@pytest.mark.redismod
 def test_resp_dollar(client):
-
     data = {
         "L1": {
             "a": {
@@ -1075,7 +1053,7 @@ def test_resp_dollar(client):
     client.json().set("doc1", "$", data)
     # Test multi
     res = client.json().resp("doc1", "$..a")
-    assert res == [
+    resp2 = [
         [
             "{",
             "A1_B1",
@@ -1131,10 +1109,67 @@ def test_resp_dollar(client):
             ["{", "A2_B4_C1", "bar"],
         ],
     ]
+    resp3 = [
+        [
+            "{",
+            "A1_B1",
+            10,
+            "A1_B2",
+            "false",
+            "A1_B3",
+            [
+                "{",
+                "A1_B3_C1",
+                None,
+                "A1_B3_C2",
+                [
+                    "[",
+                    "A1_B3_C2_D1_1",
+                    "A1_B3_C2_D1_2",
+                    -19.5,
+                    "A1_B3_C2_D1_4",
+                    "A1_B3_C2_D1_5",
+                    ["{", "A1_B3_C2_D1_6_E1", "true"],
+                ],
+                "A1_B3_C3",
+                ["[", 1],
+            ],
+            "A1_B4",
+            ["{", "A1_B4_C1", "foo"],
+        ],
+        [
+            "{",
+            "A2_B1",
+            20,
+            "A2_B2",
+            "false",
+            "A2_B3",
+            [
+                "{",
+                "A2_B3_C1",
+                None,
+                "A2_B3_C2",
+                [
+                    "[",
+                    "A2_B3_C2_D1_1",
+                    "A2_B3_C2_D1_2",
+                    -37.5,
+                    "A2_B3_C2_D1_4",
+                    "A2_B3_C2_D1_5",
+                    ["{", "A2_B3_C2_D1_6_E1", "false"],
+                ],
+                "A2_B3_C3",
+                ["[", 2],
+            ],
+            "A2_B4",
+            ["{", "A2_B4_C1", "bar"],
+        ],
+    ]
+    assert_resp_response(client, res, resp2, resp3)
 
     # Test single
-    resSingle = client.json().resp("doc1", "$.L1.a")
-    assert resSingle == [
+    res = client.json().resp("doc1", "$.L1.a")
+    resp2 = [
         [
             "{",
             "A1_B1",
@@ -1163,6 +1198,36 @@ def test_resp_dollar(client):
             ["{", "A1_B4_C1", "foo"],
         ]
     ]
+    resp3 = [
+        [
+            "{",
+            "A1_B1",
+            10,
+            "A1_B2",
+            "false",
+            "A1_B3",
+            [
+                "{",
+                "A1_B3_C1",
+                None,
+                "A1_B3_C2",
+                [
+                    "[",
+                    "A1_B3_C2_D1_1",
+                    "A1_B3_C2_D1_2",
+                    -19.5,
+                    "A1_B3_C2_D1_4",
+                    "A1_B3_C2_D1_5",
+                    ["{", "A1_B3_C2_D1_6_E1", "true"],
+                ],
+                "A1_B3_C3",
+                ["[", 1],
+            ],
+            "A1_B4",
+            ["{", "A1_B4_C1", "foo"],
+        ]
+    ]
+    assert_resp_response(client, res, resp2, resp3)
 
     # Test missing path
     client.json().resp("doc1", "$.nowhere")
@@ -1172,9 +1237,7 @@ def test_resp_dollar(client):
     client.json().resp("non_existing_doc", "$..a")
 
 
-@pytest.mark.redismod
 def test_arrindex_dollar(client):
-
     client.json().set(
         "store",
         "$",
@@ -1217,10 +1280,13 @@ def test_arrindex_dollar(client):
         },
     )
 
-    assert client.json().get("store", "$.store.book[?(@.price<10)].size") == [
-        [10, 20, 30, 40],
-        [5, 10, 20, 30],
-    ]
+    assert_resp_response(
+        client,
+        client.json().get("store", "$.store.book[?(@.price<10)].size"),
+        [[10, 20, 30, 40], [5, 10, 20, 30]],
+        [[[10, 20, 30, 40], [5, 10, 20, 30]]],
+    )
+
     assert client.json().arrindex(
         "store", "$.store.book[?(@.price<10)].size", "20"
     ) == [-1, -1]
@@ -1241,13 +1307,14 @@ def test_arrindex_dollar(client):
         ],
     )
 
-    assert client.json().get("test_num", "$..arr") == [
+    res = [
         [0, 1, 3.0, 3, 2, 1, 0, 3],
         [5, 4, 3, 2, 1, 0, 1, 2, 3.0, 2, 4, 5],
         [2, 4, 6],
         "3",
         [],
     ]
+    assert_resp_response(client, client.json().get("test_num", "$..arr"), res, [res])
 
     assert client.json().arrindex("test_num", "$..arr", 3) == [3, 2, -1, None, -1]
 
@@ -1273,13 +1340,14 @@ def test_arrindex_dollar(client):
             ],
         ],
     )
-    assert client.json().get("test_string", "$..arr") == [
+    res = [
         ["bazzz", "bar", 2, "baz", 2, "ba", "baz", 3],
         [None, "baz2", "buzz", 2, 1, 0, 1, "2", "baz", 2, 4, 5],
         ["baz2", 4, 6],
         "3",
         [],
     ]
+    assert_resp_response(client, client.json().get("test_string", "$..arr"), res, [res])
 
     assert client.json().arrindex("test_string", "$..arr", "baz") == [
         3,
@@ -1365,13 +1433,14 @@ def test_arrindex_dollar(client):
             ],
         ],
     )
-    assert client.json().get("test_None", "$..arr") == [
+    res = [
         ["bazzz", "None", 2, None, 2, "ba", "baz", 3],
         ["zaz", "baz2", "buzz", 2, 1, 0, 1, "2", None, 2, 4, 5],
         ["None", 4, 6],
         None,
         [],
     ]
+    assert_resp_response(client, client.json().get("test_None", "$..arr"), res, [res])
 
     # Test with none-scalar value
     assert client.json().arrindex(
@@ -1393,7 +1462,6 @@ def test_arrindex_dollar(client):
     assert client.json().arrindex("test_None", "..nested2_not_found.arr", "None") == 0
 
 
-@pytest.mark.redismod
 def test_decoders_and_unstring():
     assert unstring("4") == 4
     assert unstring("45.55") == 45.55
@@ -1404,7 +1472,6 @@ def test_decoders_and_unstring():
     assert decode_list(["hello", b"world"]) == ["hello", "world"]
 
 
-@pytest.mark.redismod
 def test_custom_decoder(client):
     import json
 
@@ -1412,7 +1479,7 @@ def test_custom_decoder(client):
 
     cj = client.json(encoder=ujson, decoder=ujson)
     assert cj.set("foo", Path.root_path(), "bar")
-    assert "bar" == cj.get("foo")
+    assert_resp_response(client, cj.get("foo"), "bar", [["bar"]])
     assert cj.get("baz") is None
     assert 1 == cj.delete("foo")
     assert client.exists("foo") == 0
@@ -1420,7 +1487,6 @@ def test_custom_decoder(client):
     assert not isinstance(cj.__decoder__, json.JSONDecoder)
 
 
-@pytest.mark.redismod
 def test_set_file(client):
     import json
     import tempfile
@@ -1434,12 +1500,11 @@ def test_set_file(client):
     nojsonfile.write(b"Hello World")
 
     assert client.json().set_file("test", Path.root_path(), jsonfile.name)
-    assert client.json().get("test") == obj
+    assert_resp_response(client, client.json().get("test"), obj, [[obj]])
     with pytest.raises(json.JSONDecodeError):
         client.json().set_file("test2", Path.root_path(), nojsonfile.name)
 
 
-@pytest.mark.redismod
 def test_set_path(client):
     import json
     import tempfile
@@ -1456,4 +1521,7 @@ def test_set_path(client):
 
     result = {jsonfile: True, nojsonfile: False}
     assert client.json().set_path(Path.root_path(), root) == result
-    assert client.json().get(jsonfile.rsplit(".")[0]) == {"hello": "world"}
+    res = {"hello": "world"}
+    assert_resp_response(
+        client, client.json().get(jsonfile.rsplit(".")[0]), res, [[res]]
+    )
