@@ -84,7 +84,7 @@ start_server {tags {"zset"}} {
             set err
         } {ERR*}
 
-        test "ZADD NX with non exisitng key" {
+        test "ZADD NX with non existing key" {
             r del ztmp
             r zadd ztmp nx 10 x 20 y 30 z
             assert {[r zcard ztmp] == 3}
@@ -388,7 +388,7 @@ start_server {tags {"zset"}} {
                               0 omega}
         }
 
-        test "ZRANGEBYLEX/ZREVRANGEBYLEX/ZLEXCOUNT basics" {
+        test "ZRANGEBYLEX/ZREVRANGEBYLEX/ZCOUNT basics" {
             create_default_lex_zset
 
             # inclusive range
@@ -416,22 +416,6 @@ start_server {tags {"zset"}} {
             assert_equal {} [r zrevrangebylex zset \[elez \[elex]
             assert_equal {} [r zrevrangebylex zset (hill (omega]
         }
-        
-        test "ZLEXCOUNT advanced" {
-            create_default_lex_zset
-    
-            assert_equal 9 [r zlexcount zset - +]
-            assert_equal 0 [r zlexcount zset + -]
-            assert_equal 0 [r zlexcount zset + \[c]
-            assert_equal 0 [r zlexcount zset \[c -]
-            assert_equal 8 [r zlexcount zset \[bar +]
-            assert_equal 5 [r zlexcount zset \[bar \[foo]
-            assert_equal 4 [r zlexcount zset \[bar (foo]
-            assert_equal 4 [r zlexcount zset (bar \[foo]
-            assert_equal 3 [r zlexcount zset (bar (foo]
-            assert_equal 5 [r zlexcount zset - (foo]
-            assert_equal 1 [r zlexcount zset (maxstring +]
-        }
 
         test "ZRANGEBYSLEX with LIMIT" {
             create_default_lex_zset
@@ -664,6 +648,75 @@ start_server {tags {"zset"}} {
                 }
             }
         }
+
+        test "Basic ZPOP with a single key - $encoding" {
+            r del zset
+            assert_equal {} [r zpopmin zset]
+            create_zset zset {-1 a 1 b 2 c 3 d 4 e}
+            assert_equal {a -1} [r zpopmin zset]
+            assert_equal {b 1} [r zpopmin zset]
+            assert_equal {e 4} [r zpopmax zset]
+            assert_equal {d 3} [r zpopmax zset]
+            assert_equal {c 2} [r zpopmin zset]
+            assert_equal 0 [r exists zset]
+            r set foo bar
+            assert_error "*WRONGTYPE*" {r zpopmin foo}
+        }
+
+        test "ZPOP with count - $encoding" {
+            r del z1 z2 z3 foo
+            r set foo bar
+            assert_equal {} [r zpopmin z1 2]
+            assert_error "*WRONGTYPE*" {r zpopmin foo 2}
+            create_zset z1 {0 a 1 b 2 c 3 d}
+            assert_equal {a 0 b 1} [r zpopmin z1 2]
+            assert_equal {d 3 c 2} [r zpopmax z1 2]
+        }
+
+        test "BZPOP with a single existing sorted set - $encoding" {
+            set rd [redis_deferring_client]
+            create_zset zset {0 a 1 b 2 c}
+
+            $rd bzpopmin zset 5
+            assert_equal {zset a 0} [$rd read]
+            $rd bzpopmin zset 5
+            assert_equal {zset b 1} [$rd read]
+            $rd bzpopmax zset 5
+            assert_equal {zset c 2} [$rd read]
+            assert_equal 0 [r exists zset]
+        }
+
+        test "BZPOP with multiple existing sorted sets - $encoding" {
+            set rd [redis_deferring_client]
+            create_zset z1 {0 a 1 b 2 c}
+            create_zset z2 {3 d 4 e 5 f}
+
+            $rd bzpopmin z1 z2 5
+            assert_equal {z1 a 0} [$rd read]
+            $rd bzpopmax z1 z2 5
+            assert_equal {z1 c 2} [$rd read]
+            assert_equal 1 [r zcard z1]
+            assert_equal 3 [r zcard z2]
+
+            $rd bzpopmax z2 z1 5
+            assert_equal {z2 f 5} [$rd read]
+            $rd bzpopmin z2 z1 5
+            assert_equal {z2 d 3} [$rd read]
+            assert_equal 1 [r zcard z1]
+            assert_equal 1 [r zcard z2]
+        }
+
+        test "BZPOP second sorted set has members - $encoding" {
+            set rd [redis_deferring_client]
+            r del z1
+            create_zset z2 {3 d 4 e 5 f}
+            $rd bzpopmax z1 z2 5
+            assert_equal {z2 f 5} [$rd read]
+            $rd bzpopmin z2 z1 5
+            assert_equal {z2 d 3} [$rd read]
+            assert_equal 0 [r zcard z1]
+            assert_equal 1 [r zcard z2]
+        }
     }
 
     basics ziplist
@@ -1041,10 +1094,121 @@ start_server {tags {"zset"}} {
             }
             assert_equal {} $err
         }
+
+        test "BZPOPMIN, ZADD + DEL should not awake blocked client" {
+            set rd [redis_deferring_client]
+            r del zset
+
+            $rd bzpopmin zset 0
+            r multi
+            r zadd zset 0 foo
+            r del zset
+            r exec
+            r del zset
+            r zadd zset 1 bar
+            $rd read
+        } {zset bar 1}
+
+        test "BZPOPMIN, ZADD + DEL + SET should not awake blocked client" {
+            set rd [redis_deferring_client]
+            r del list
+
+            r del zset
+
+            $rd bzpopmin zset 0
+            r multi
+            r zadd zset 0 foo
+            r del zset
+            r set zset foo
+            r exec
+            r del zset
+            r zadd zset 1 bar
+            $rd read
+        } {zset bar 1}
+
+        test "BZPOPMIN with same key multiple times should work" {
+            set rd [redis_deferring_client]
+            r del z1 z2
+
+            # Data arriving after the BZPOPMIN.
+            $rd bzpopmin z1 z2 z2 z1 0
+            r zadd z1 0 a
+            assert_equal [$rd read] {z1 a 0}
+            $rd bzpopmin z1 z2 z2 z1 0
+            r zadd z2 1 b
+            assert_equal [$rd read] {z2 b 1}
+
+            # Data already there.
+            r zadd z1 0 a
+            r zadd z2 1 b
+            $rd bzpopmin z1 z2 z2 z1 0
+            assert_equal [$rd read] {z1 a 0}
+            $rd bzpopmin z1 z2 z2 z1 0
+            assert_equal [$rd read] {z2 b 1}
+        }
+
+        test "MULTI/EXEC is isolated from the point of view of BZPOPMIN" {
+            set rd [redis_deferring_client]
+            r del zset
+            $rd bzpopmin zset 0
+            r multi
+            r zadd zset 0 a
+            r zadd zset 1 b
+            r zadd zset 2 c
+            r exec
+            $rd read
+        } {zset a 0}
+
+        test "BZPOPMIN with variadic ZADD" {
+            set rd [redis_deferring_client]
+            r del zset
+            if {$::valgrind} {after 100}
+            $rd bzpopmin zset 0
+            if {$::valgrind} {after 100}
+            assert_equal 2 [r zadd zset -1 foo 1 bar]
+            if {$::valgrind} {after 100}
+            assert_equal {zset foo -1} [$rd read]
+            assert_equal {bar} [r zrange zset 0 -1]
+        }
+
+        test "BZPOPMIN with zero timeout should block indefinitely" {
+            set rd [redis_deferring_client]
+            r del zset
+            $rd bzpopmin zset 0
+            after 1000
+            r zadd zset 0 foo
+            assert_equal {zset foo 0} [$rd read]
+        }
     }
 
     tags {"slow"} {
         stressers ziplist
         stressers skiplist
     }
+
+    test {ZSET skiplist order consistency when elements are moved} {
+        set original_max [lindex [r config get zset-max-ziplist-entries] 1]
+        r config set zset-max-ziplist-entries 0
+        for {set times 0} {$times < 10} {incr times} {
+            r del zset
+            for {set j 0} {$j < 1000} {incr j} {
+                r zadd zset [randomInt 50] ele-[randomInt 10]
+            }
+
+            # Make sure that element ordering is correct
+            set prev_element {}
+            set prev_score -1
+            foreach {element score} [r zrange zset 0 -1 WITHSCORES] {
+                # Assert that elements are in increasing ordering
+                assert {
+                    $prev_score < $score ||
+                    ($prev_score == $score &&
+                     [string compare $prev_element $element] == -1)
+                }
+                set prev_element $element
+                set prev_score $score
+            }
+        }
+        r config set zset-max-ziplist-entries $original_max
+    }
 }
